﻿#pragma once


#include <cigui/utils/List.hpp>
#include <cigui/core/Layout.hpp>
#include <cigui/core/RenderCall.hpp>
#include <memory>


namespace cig
{
	class View {
	protected:
		List<RenderCall> m_RenderCalls;
		Layout m_Layout;

	public:
		virtual ~View() = default;

		[[nodiscard]] const List<RenderCall> &renderCalls() const { return m_RenderCalls; }
		std::unique_ptr<View> content;

		virtual bool update() {
			if (content)
				return content->update();
			return false;
		}

		void draw() {
			if (!m_RenderCalls.empty()) {
				m_RenderCalls.clear();
			}
			content = std::unique_ptr<View>(body());
			if (!content) {
				return;
			}
			content->draw();
			auto &contentRenderCalls = content->renderCalls();
			m_RenderCalls.expand(contentRenderCalls);
		}

		virtual View *body() = 0;
	};
}