#pragma once

#include <functional>
#include <memory>
namespace cig
{
	template <typename T, size_t growth_scalar = 1, size_t growth_summand = 3>
	class List
	{
		std::unique_ptr<T> m_Data;
		size_t m_Size = 0;
		size_t m_Capacity;

	protected:
		void reserve(const size_t capacity);

	public:
		explicit List(const size_t capacity = 3) : m_Capacity(capacity) { reserve(capacity); }

		void own(T* data, const size_t size)
		{
			m_Data = data;
			m_Size = size;
		}

		void copy(T* data, const size_t size)
		{
			m_Data = std::make_unique<T[]>(size);
			std::copy(data, data + size, m_Data.get());
			m_Size = size;
		}

		[[nodiscard]] size_t size() const { return m_Size; }

		T& operator[](size_t index) { return m_Data.get()[index]; }

		const T& operator[](size_t index) const { return m_Data.get()[index]; }

		void need(const size_t additional_size)
		{
			if (m_Size + additional_size > m_Capacity)
				reserve(m_Capacity + additional_size);
		}

		[[nodiscard]] bool empty() const { return m_Size == 0; }

		void clear() { m_Size = 0; }

		void push_back(const T& value)
		{
			if (m_Size >= m_Capacity)
				reserve(m_Capacity * growth_scalar + growth_summand);
			m_Data.get()[m_Size++] = value;
		}

		template <typename... Args>
		void emplace_back(Args&&... args)
		{
			if (m_Size >= m_Capacity)
				reserve(m_Capacity * growth_scalar + growth_summand);
			m_Data.get()[m_Size++] = T(std::forward<Args>(args)...);
		}

		void expand(const List<T>& other)
		{
			need(other.size());
			std::copy(other.m_Data.get(), other.m_Data.get() + other.size(), m_Data.get() + m_Size);
			m_Size += other.size();
		}

		template <typename Lambda>
		void iterate(Lambda&& lambda) const
		{
			for (size_t i = 0; i < m_Size; i++)
				lambda(m_Data.get()[i]);
		}
	};
}
#include <cigui/utils/List.inl>